"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnnounceService = void 0;
/*
 * Copyright 2021 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const symbol_sdk_1 = require("symbol-sdk");
const logger_1 = require("../logger");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const BootstrapUtils_1 = require("./BootstrapUtils");
const logger = LoggerFactory_1.default.getLogger(logger_1.LogType.System);
class AnnounceService {
    async announce(repositoryFactory, presetData, transactionNodes, generationHash, tokenAmount = 'some') {
        if (!transactionNodes.length) {
            logger.info(`There are no transactions to announce...`);
            return;
        }
        const transactionRepository = repositoryFactory.createTransactionRepository();
        const transactionService = new symbol_sdk_1.TransactionService(transactionRepository, repositoryFactory.createReceiptRepository());
        const listener = repositoryFactory.createListener();
        await listener.open();
        const faucetUrl = presetData.faucetUrl;
        const signedTransactionObservable = rxjs_1.from(transactionNodes).pipe(operators_1.mergeMap(({ node, transactions }) => {
            if (!node.main) {
                throw new Error('CA account is required!');
            }
            const account = symbol_sdk_1.Account.createFromPrivateKey(node.main.privateKey, presetData.networkType);
            const noFundsMessage = faucetUrl
                ? `Does your node signing address have any network coin? Send ${tokenAmount} tokens to ${account.address.plain()} via ${faucetUrl}/?recipient=${account.address.plain()}`
                : `Does your node signing address have any network coin? Send ${tokenAmount} tokens to ${account.address.plain()} .`;
            const accountInfoObservable = repositoryFactory.createAccountRepository().getAccountInfo(account.address);
            const multisigInfoObservable = repositoryFactory
                .createMultisigRepository()
                .getMultisigAccountInfo(account.address)
                .pipe(operators_1.map((multisigAccount) => {
                return { multisigAccount: multisigAccount };
            }), operators_1.catchError(() => {
                return rxjs_1.of({ multisigAccount: undefined });
            }));
            return rxjs_1.combineLatest([accountInfoObservable, multisigInfoObservable]).pipe(operators_1.mergeMap(([a, mutlsigiInfo]) => {
                const multisigAccount = mutlsigiInfo.multisigAccount;
                if (multisigAccount) {
                    logger.error(`Node signing account ${multisigAccount.accountAddress.plain()} is a multisig account! This command doesn't support multisig accounts!`);
                    return rxjs_1.EMPTY;
                }
                const currencyMosaicIdHex = BootstrapUtils_1.BootstrapUtils.toHex(presetData.currencyMosaicId);
                const mosaic = a.mosaics.find((m) => BootstrapUtils_1.BootstrapUtils.toHex(m.id.toHex()) === currencyMosaicIdHex);
                if (!mosaic || mosaic.amount.compare(symbol_sdk_1.UInt64.fromUint(0)) < 1) {
                    logger.error(`Node signing account ${account.address.plain()} does not have enough currency. Mosaic id: ${currencyMosaicIdHex}. \n\n${noFundsMessage}`);
                    return rxjs_1.EMPTY;
                }
                return rxjs_1.from(transactions.map((t) => account.sign(t, generationHash)));
            }), operators_1.catchError((e) => {
                logger.error(`Node signing account ${account.address.plain()} is not valid. ${e.message}. \n\n${noFundsMessage}`);
                return rxjs_1.EMPTY;
            }));
        }));
        const announceCalls = signedTransactionObservable.pipe(operators_1.mergeMap((signedTransaction) => {
            return transactionService.announce(signedTransaction, listener).pipe(operators_1.map((completedTransaction) => {
                var _a, _b;
                const message = `Transaction ${completedTransaction.type} ${(_a = completedTransaction.transactionInfo) === null || _a === void 0 ? void 0 : _a.hash} - signer ${(_b = completedTransaction.signer) === null || _b === void 0 ? void 0 : _b.address.plain()} has been confirmed`;
                logger.info(message);
                return message;
            }), operators_1.catchError((e) => {
                const message = `Transaction ${signedTransaction.type} ${signedTransaction.hash} - signer ${signedTransaction.getSignerAddress().plain()} failed!! ` + e.message;
                logger.error(message);
                return rxjs_1.of(message);
            }));
        }));
        await announceCalls.pipe(operators_1.toArray()).toPromise();
        listener.close();
    }
}
exports.AnnounceService = AnnounceService;
